import { describe, it, expect, vi } from 'vitest';
import { useRequired } from '../useRequired';

vi.mock('../useValidator', () => ({
    useValidator: () => ({
        validations: () => ({
            required: vi.fn((isRequired, val) => {
                if (!isRequired) return true;
                return val !== null && val !== undefined && val !== '';
            }),
        }),
    }),
}));

describe('useRequired', () => {
    it('should detect required when attr is boolean true', () => {
        const attrs = { required: true };
        const { isRequired } = useRequired(attrs);
        expect(isRequired).toBe(true);
    });

    it('should detect required when attr is boolean false', () => {
        const attrs = { required: false };
        const { isRequired } = useRequired(attrs);
        expect(isRequired).toBe(false);
    });

    it('should detect required when attr exists without value', () => {
        const attrs = { required: '' };
        const { isRequired } = useRequired(attrs);
        expect(isRequired).toBe(true);
    });

    it('should return false when required attr does not exist', () => {
        const attrs = { someOtherAttr: 'value' };
        const { isRequired } = useRequired(attrs);
        expect(isRequired).toBe(false);
    });

    describe('requiredFieldRule', () => {
        it('should validate required field with value', () => {
            const attrs = { required: true };
            const { requiredFieldRule } = useRequired(attrs);
            expect(requiredFieldRule('some value')).toBe(true);
        });

        it('should validate required field with empty value', () => {
            const attrs = { required: true };
            const { requiredFieldRule } = useRequired(attrs);
            expect(requiredFieldRule('')).toBe(false);
        });

        it('should pass validation when field is not required', () => {
            const attrs = { required: false };
            const { requiredFieldRule } = useRequired(attrs);
            expect(requiredFieldRule('')).toBe(true);
        });

        it('should handle null and undefined values', () => {
            const attrs = { required: true };
            const { requiredFieldRule } = useRequired(attrs);
            expect(requiredFieldRule(null)).toBe(false);
            expect(requiredFieldRule(undefined)).toBe(false);
        });
    });
});
