import { describe, it, expect, vi, afterEach } from 'vitest';
import axios from 'axios';
import { getAgencies } from 'src/pages/Route/Agency/composables/getAgencies';

vi.mock('axios');
const response = { data: [{ agencyModeFk: 'Agency1' }, { agencyModeFk: 'Agency2' }] };
axios.get.mockResolvedValue(response);

describe('getAgencies', () => {
    afterEach(() => {
        vi.clearAllMocks();
    });

    const generateParams = (formData, filter = {}) => ({
        params: {
            filter: JSON.stringify(filter),
            warehouseFk: formData.warehouseId,
            addressFk: formData.addressId,
            landed: formData.landed,
        },
    });

    it('should fetch agencies data with correct parameters for valid formData', async () => {
        const formData = {
            warehouseId: '123',
            addressId: '456',
            landed: 'true',
        };
        const filter = {
            fields: ['nickname', 'street', 'city', 'id'],
            where: { isActive: true },
            order: 'nickname ASC',
        };

        await getAgencies(formData, null, filter);

        expect(axios.get).toHaveBeenCalledWith('Agencies/getAgenciesWithWarehouse', generateParams(formData, filter));
    });

    it('should not call API when formData is missing required landed field', async () => {
        const formData = { warehouseId: '123', addressId: '456' };

        await getAgencies(formData);

        expect(axios.get).not.toHaveBeenCalled();
    });

    it('should not call API when formData is missing required addressId field', async () => {
        const formData = { warehouseId: '123', landed: 'true' };

        await getAgencies(formData);

        expect(axios.get).not.toHaveBeenCalled();
    });

    it('should not call API when formData is missing required warehouseId field', async () => {
        const formData = { addressId: '456', landed: 'true' };

        await getAgencies(formData);

        expect(axios.get).not.toHaveBeenCalled();
    });

    it('should return options and agency when default agency is found', async () => {
        const formData = { warehouseId: '123', addressId: '456', landed: 'true' };
        const client = { defaultAddress: { agencyModeFk: 'Agency1' } };
        
        const { options, agency } = await getAgencies(formData, client);
        
        expect(options).toEqual(response.data);
        expect(agency).toEqual(response.data[0]);
      });

      it('should return options and agency when client is not provided', async () => {
        const formData = { warehouseId: '123', addressId: '456', landed: 'true' };
        
        const { options, agency } = await getAgencies(formData);
        
        expect(options).toEqual(response.data);
        expect(agency).toBeNull();
      });
});
