import { setActivePinia, createPinia } from 'pinia';
import { describe, beforeEach, afterEach, it, expect, vi, beforeAll } from 'vitest';
import { useNavigationStore } from '../useNavigationStore';
import axios from 'axios';

let store;

vi.mock('src/router/modules', () => [
    { name: 'Item', meta: {} },
    { name: 'Shelving', meta: {} },
    { name: 'Order', meta: {} },
]);

vi.mock('src/filters', () => ({
    toLowerCamel: vi.fn((name) => name.toLowerCase()),
}));

const modulesMock = [
    {
        name: 'Item',
        children: null,
        title: 'globals.pageTitles.undefined',
        icon: undefined,
        module: 'item',
        isPinned: true,
    },
    {
        name: 'Shelving',
        children: null,
        title: 'globals.pageTitles.undefined',
        icon: undefined,
        module: 'shelving',
        isPinned: false,
    },
    {
        name: 'Order',
        children: null,
        title: 'globals.pageTitles.undefined',
        icon: undefined,
        module: 'order',
        isPinned: false,
    },
];

const pinnedModulesMock = [
    {
        name: 'Item',
        children: null,
        title: 'globals.pageTitles.undefined',
        icon: undefined,
        module: 'item',
        isPinned: true,
    },
];

describe('useNavigationStore', () => {
    beforeEach(() => {
        setActivePinia(createPinia());
        vi.spyOn(axios, 'get').mockResolvedValue({ data: true });
        store = useNavigationStore();
        store.getModules = vi.fn().mockReturnValue({
            value: modulesMock,
        });
        store.getPinnedModules = vi.fn().mockReturnValue({
            value: pinnedModulesMock,
        });
    });
    afterEach(() => {
        vi.clearAllMocks();
    });

    it('should return modules with correct structure', () => {
        const store = useNavigationStore();
        const modules = store.getModules();

        expect(modules.value).toEqual(modulesMock);
    });

    it('should return pinned modules', () => {
        const store = useNavigationStore();
        const pinnedModules = store.getPinnedModules();

        expect(pinnedModules.value).toEqual(pinnedModulesMock);
    });

    it('should toggle pinned modules', () => {
        const store = useNavigationStore();

        store.togglePinned('item');
        store.togglePinned('shelving');
        expect(store.pinnedModules).toEqual(['item', 'shelving']);

        store.togglePinned('item');
        expect(store.pinnedModules).toEqual(['shelving']);
    });

    it('should fetch pinned modules', async () => {
        vi.spyOn(axios, 'get').mockResolvedValue({
            data: [{ id: 1, workerFk: 9, moduleFk: 'order', position: 1 }],
        });
        const store = useNavigationStore();
        await store.fetchPinned();

        expect(store.pinnedModules).toEqual(['order']);
    });

    it('should add menu item correctly', () => {
        const store = useNavigationStore();
        const module = 'customer';
        const parent = [];
        const route = {
            name: 'customer',
            title: 'Customer',
            icon: 'customer',
            meta: {
                keyBinding: 'ctrl+shift+c',
                name: 'customer',
                title: 'Customer',
                icon: 'customer',
                menu: 'customer',
                menuChildren: [{ name: 'customer', title: 'Customer', icon: 'customer' }],
            },
        };

        const result = store.addMenuItem(module, route, parent);
        const expectedItem = {
            children: [
                {
                    icon: 'customer',
                    name: 'customer',
                    title: 'globals.pageTitles.Customer',
                },
            ],
            icon: 'customer',
            keyBinding: 'ctrl+shift+c',
            name: 'customer',
            title: 'globals.pageTitles.Customer',
        };
        expect(result).toEqual(expectedItem);
        expect(parent.length).toBe(1);
        expect(parent).toEqual([expectedItem]);
    });

    it('should not add menu item if condition is not met', () => {
        const store = useNavigationStore();
        const module = 'testModule';
        const route = { meta: { hidden: true, menuchildren: {} } };
        const parent = [];
        const result = store.addMenuItem(module, route, parent);
        expect(result).toBeUndefined();
        expect(parent.length).toBe(0);
    });
});
